@@ -20,14 +20,9 @@ import org.gnu.glib.Type;
 public class Bin extends Container 
 {
 	protected Bin(int handle) {
-	    setHandle(handle);
-	    initializeEventHandlers();
+	    super(handle);
 	}
 
-	protected Bin() {}
-
-	protected void noopInit() {}
-	
 	/**
 	 * Retruns the child of this Bin or null if the bin contains no
 	 * child.
