@@ -22,7 +22,7 @@ public class FontButton extends Button {
 	
 	// for libglade
 	public FontButton(int hndl) {
-		setHandle(hndl);
+		super(hndl);
 		initializeEventHandlers();
 	}
 	
@@ -30,10 +30,8 @@ public class FontButton extends Button {
 	 * Construct a new FontButton widget
 	 */
 	public FontButton(){
-		if (-1 == getHandle()) {
-			setHandle(gtk_font_button_new());
-			initializeEventHandlers();
-		}
+		super(gtk_font_button_new());
+		initializeEventHandlers();
 	}
 
 	/**
@@ -44,7 +42,7 @@ public class FontButton extends Button {
 	 * font selection dialog.
 	 */
 	public FontButton(String fontName) {
-		setHandle(gtk_font_button_new_with_font(fontName));
+		super(gtk_font_button_new_with_font(fontName));
 		initializeEventHandlers();
 	}
 	
@@ -214,9 +212,6 @@ public class FontButton extends Button {
 		fireFontButtonEvent(new FontButtonEvent(this, FontButtonEvent.Type.FONT_SET));
 	}
 
-	/** Are events initialized? */
-	private boolean eventsInitialized = false;
-
 	protected void initializeEventHandlers() {
 		if (!eventsInitialized) {
 			super.initializeEventHandlers();
@@ -226,11 +221,15 @@ public class FontButton extends Button {
 	}
 
 	public Class getEventListenerClass(String signal) {
-		return evtMap.getListenerClass(signal);
+		Class cls = evtMap.getEventListenerClass(signal);
+		if (cls == null) cls = super.getEventListenerClass(signal);
+		return cls;
 	}
 
 	public GtkEventType getEventType(String signal) {
-		return evtMap.getEventType(signal);
+		GtkEventType et = evtMap.getEventType(signal);
+		if (et == null) et = super.getEventType(signal);
+		return et;
 	}
 
 	private static EventMap evtMap = new EventMap();
