@@ -36,24 +36,23 @@ public class SimpleList extends Widget {
 	 * Constructs a SimpleList object.
 	 */
 	public SimpleList() {
-		noopInit();
+		super(init());
+		dataBlock = new DataColumnString();
+		store = new ListStore(new DataColumn[] { dataBlock });
+		view = new TreeView(getHandle());
+		view.setModel(store);
+		column = new TreeViewColumn();
+		renderer = new CellRendererText();
+		column.packStart(renderer, false);
+		column.addAttributeMapping(renderer, CellRendererText.Attribute.TEXT, dataBlock);
+		view.appendColumn(column);
+		view.setHeadersVisible(false);
+		size = 0;
 	}
-
-	protected void noopInit() {
-		if (-1 == getHandle()) {
-			dataBlock = new DataColumnString();
-			store = new ListStore(new DataColumn[] { dataBlock });
-			view = new TreeView(store);
-			column = new TreeViewColumn();
-			renderer = new CellRendererText();
-			column.packStart(renderer, false);
-			column.addAttributeMapping(renderer, CellRendererText.Attribute.TEXT, dataBlock);
-			view.appendColumn(column);
-			view.setHeadersVisible(false);
-			this.size = 0;
-
-			this.setHandle(view.getHandle());
-		}
+	
+	private static int init() {
+		TreeView v = new TreeView();
+		return v.getHandle();
 	}
 
 	/**
