@@ -29,21 +29,16 @@ public class Table extends Container {
 	 * the size of the cell containing the largest widget.
 	 */
 	public Table(int rows, int columns, boolean homogenous) {
-		setHandle(gtk_table_new(rows, columns, homogenous));
-		initializeEventHandlers();
+		super(gtk_table_new(rows, columns, homogenous));
 	}
 
 	/**
 	 * Construct a new Table from a handle to a native resource.
 	 */
 	public Table(int handle) {
-		setHandle(handle);
-		initializeEventHandlers();
+		super(handle);
 	}
 	
-	protected void noopInit() {
-	}
-
 	/**
 	 * Changes the size of the table after it has been created. The parameters
 	 * define the requested total number of rows/colums after the resize.
