@@ -96,14 +96,8 @@ public class TextView extends Container {
 	 * a default one will be created for you.
 	 */
 	public TextView() {
-		noopInit();
-	}
-
-	protected void noopInit() {
-		if (-1 == getHandle()) {
-			setHandle(gtk_text_view_new());
-			initializeEventHandlers();
-		}
+		super(gtk_text_view_new());
+		initializeEventHandlers();
 	}
 
 	/**
@@ -112,7 +106,7 @@ public class TextView extends Container {
 	 * @param buffer Buffer to use
 	 */
 	public TextView(TextBuffer buffer) {
-		setHandle(gtk_text_view_new_with_buffer(buffer.getHandle()));
+		super(gtk_text_view_new_with_buffer(buffer.getHandle()));
 		initializeEventHandlers();
 	}
 
@@ -120,7 +114,7 @@ public class TextView extends Container {
 	 * Construct a TextView from a handle to a native resource.
 	 */
 	public TextView(int handle) {
-		setHandle(handle);
+		super(handle);
 		initializeEventHandlers();
 	}
 
@@ -389,9 +383,6 @@ public class TextView extends Container {
 		return gtk_text_view_get_overwrite (getHandle());
 	}
 
-	/** Are events initialized? */
-	private boolean eventsInitialized = false;
-
 	/**
 	 * Initialize this widget to receive its' own events so it can
 	 * propogate them to the listeners.
@@ -405,11 +396,15 @@ public class TextView extends Container {
 	}
 
 	public Class getEventListenerClass(String signal) {
-		return evtMap.getListenerClass(signal);
+		Class cls = evtMap.getEventListenerClass(signal);
+		if (cls == null) cls = super.getEventListenerClass(signal);
+		return cls;
 	}
 
 	public GtkEventType getEventType(String signal) {
-		return evtMap.getEventType(signal);
+		GtkEventType et = evtMap.getEventType(signal);
+		if (et == null) et = super.getEventType(signal);
+		return et;
 	}
 
 	private static EventMap evtMap = new EventMap();
