@@ -21,20 +21,17 @@ import org.gnu.gtk.event.ToggleToolButtonListener;
 public class ToggleToolButton extends ToolButton {
 
 	public ToggleToolButton() {
-		super();
-		if (-1 == getHandle()) {
-			setHandle(gtk_toggle_tool_button_new());
-			initializeEventHandlers();
-		}
+		super(gtk_toggle_tool_button_new());
+		initializeEventHandlers();
 	}
 	
 	public ToggleToolButton(String stockId) {
-		setHandle(gtk_toggle_tool_button_new_from_stock(stockId));
+		super(gtk_toggle_tool_button_new_from_stock(stockId));
 		initializeEventHandlers();
 	}
 	
 	public ToggleToolButton(int hndl) {
-		setHandle(hndl);
+		super(hndl);
 		initializeEventHandlers();
 	}
 	
@@ -101,9 +98,6 @@ public class ToggleToolButton extends ToolButton {
 		fireToggleToolButtonEvent(evt);
 	}
 
-	/** Are events initialized? */
-	private boolean eventsInitialized = false;
-
 	protected void initializeEventHandlers() {
 		if (!eventsInitialized) {
 			super.initializeEventHandlers();
@@ -113,11 +107,15 @@ public class ToggleToolButton extends ToolButton {
 	}
 
 	public Class getEventListenerClass(String signal) {
-		return evtMap.getListenerClass(signal);
+		Class cls = evtMap.getEventListenerClass(signal);
+		if (cls == null) cls = super.getEventListenerClass(signal);
+		return cls;
 	}
 
 	public GtkEventType getEventType(String signal) {
-		return evtMap.getEventType(signal);
+		GtkEventType et = evtMap.getEventType(signal);
+		if (et == null) et = super.getEventType(signal);
+		return et;
 	}
 
 	private static EventMap evtMap = new EventMap();
