@@ -22,22 +22,24 @@ import org.gnu.gtk.event.ToolButtonListener;
 public class ToolButton extends ToolItem {
 	
 	public ToolButton(Widget iconWidget, String label) {
+		super(init(iconWidget, label));
+		initializeEventHandlers();
+	}
+	
+	private static int init(Widget iconWidget, String label) {
 		int iconHndl = 0;
 		if (null != iconWidget)
 			iconHndl = iconWidget.getHandle();
-		setHandle(gtk_tool_button_new(iconHndl, label));
-		initializeEventHandlers();
+		return gtk_tool_button_new(iconHndl, label);
 	}
 	
 	public ToolButton(String stockId) {
-		setHandle(gtk_tool_button_new_from_stock(stockId));
+		super(gtk_tool_button_new_from_stock(stockId));
 		initializeEventHandlers();
 	}
 	
-	protected ToolButton() {}
-	
 	public ToolButton(int hndl) {
-		setHandle(hndl);
+		super(hndl);
 		initializeEventHandlers();
 	}
 	
@@ -146,9 +148,6 @@ public class ToolButton extends ToolItem {
 		fireToolButtonEvent(evt);
 	}
 
-	/** Are events initialized? */
-	private boolean eventsInitialized = false;
-
 	protected void initializeEventHandlers() {
 		if (!eventsInitialized) {
 			super.initializeEventHandlers();
@@ -158,11 +157,15 @@ public class ToolButton extends ToolItem {
 	}
 
 	public Class getEventListenerClass(String signal) {
-		return evtMap.getListenerClass(signal);
+		Class cls = evtMap.getEventListenerClass(signal);
+		if (cls == null) cls = super.getEventListenerClass(signal);
+		return cls;
 	}
 
 	public GtkEventType getEventType(String signal) {
-		return evtMap.getEventType(signal);
+		GtkEventType et = evtMap.getEventType(signal);
+		if (et == null) et = super.getEventType(signal);
+		return et;
 	}
 
 	private static EventMap evtMap = new EventMap();
