@@ -30,6 +30,7 @@ import org.gnu.gdk.Rectangle;
 import org.gnu.gdk.Screen;
 import org.gnu.gdk.Window;
 import org.gnu.glib.EventMap;
+import org.gnu.glib.GObject;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.DragDestinationListener;
 import org.gnu.gtk.event.DragSourceListener;
@@ -89,42 +90,36 @@ public class Widget extends GtkObject {
 	 * @param handle The handle that represents a pointer to a native resource.
 	 */
 	public Widget(int handle) {
-		this.handle = handle;
+		super(handle);
 		initializeEventHandlers();
 	}
 
-	protected Widget() {
-	}
-
-	protected void noopInit() {
-	}
-
 	/**
 	 * Cause this Widget to be visible.
 	 */
 	public void show() {
-		Widget.gtk_widget_show(handle);
+		Widget.gtk_widget_show(getHandle());
 	}
 
 	/**
 	 * Cause this Widget and all of its' children to be visible.
 	 */
 	public void showAll() {
-		Widget.gtk_widget_show_all(handle);
+		Widget.gtk_widget_show_all(getHandle());
 	}
 
 	/**
 	 * Cause this Widget to not be visible.
 	 */
 	public void hide() {
-		Widget.gtk_widget_hide(handle);
+		Widget.gtk_widget_hide(getHandle());
 	}
 
 	/**
 	 * Cause this Widget and all of its' children to not be visible.
 	 */
 	public void hideAll() {
-		Widget.gtk_widget_hide_all(handle);
+		Widget.gtk_widget_hide_all(getHandle());
 	}
 
 	/**
@@ -132,7 +127,7 @@ public class Widget extends GtkObject {
 	 * removed from the container.  All of this widget's children are also destroyed.
 	 */
 	public void destroy() {
-		Widget.gtk_widget_destroy(handle);
+		Widget.gtk_widget_destroy(getHandle());
 	}
 
 	/**
@@ -141,14 +136,14 @@ public class Widget extends GtkObject {
 	 * then the widget will be realized automatically.
 	 */
 	public void realize() {
-		Widget.gtk_widget_realize(handle);
+		Widget.gtk_widget_realize(getHandle());
 	}
 
 	/**
 	 *  Causes the entire widget to be redrawn.
 	 */
 	public void draw() {
-		Widget.gtk_widget_queue_draw(handle);
+		Widget.gtk_widget_queue_draw(getHandle());
 	}
 
 	/**
@@ -169,7 +164,7 @@ public class Widget extends GtkObject {
 	 * @param height The height of the area to be redrawn.
 	 */
 	public void drawArea(int x, int y, int width, int height) {
-		Widget.gtk_widget_queue_draw_area(handle, x, y, width, height);
+		Widget.gtk_widget_queue_draw_area(getHandle(), x, y, width, height);
 	}
 
 	/**
@@ -180,7 +175,7 @@ public class Widget extends GtkObject {
 	 * @return False if this widget is not activatable.
 	 */
 	public boolean activate() {
-		return Widget.gtk_widget_activate(handle);
+		return Widget.gtk_widget_activate(getHandle());
 	}
 
 	/**
@@ -189,7 +184,7 @@ public class Widget extends GtkObject {
 	 * @param newParent The widget that will be the new parent.
 	 */
 	public void reparent(Widget newParent) {
-		Widget.gtk_widget_reparent(handle, newParent.getHandle());
+		Widget.gtk_widget_reparent(getHandle(), newParent.getHandle());
 	}
 
 	/**
@@ -198,7 +193,13 @@ public class Widget extends GtkObject {
 	 * @return The parent container for this widget.
 	 */
 	public Widget getParent() {
-		return new Widget(Widget.gtk_widget_get_parent(handle));
+		int hndl = gtk_widget_get_parent(getHandle());
+		if (0 == hndl)
+			return null;
+		GObject obj = retrieveGObject(hndl);
+		if (null != obj)
+			return (Widget)obj;
+		return new Widget(hndl);
 	}
 
 	/**
@@ -207,7 +208,11 @@ public class Widget extends GtkObject {
 	 * @return Returns the widget's parent window.
 	 */
 	public Window getParentWindow() {
-		return new Window(Widget.gtk_widget_get_parent_window(handle));
+		int hndl = gtk_widget_get_parent_window(getHandle());
+		GObject obj = retrieveGObject(hndl);
+		if (null != obj)
+			return (Window)obj;
+		return new Window(hndl);
 	}
 
 	/**
@@ -218,7 +223,7 @@ public class Widget extends GtkObject {
 	 */
 	public Rectangle intersect(Rectangle area) {
 		int[] rect = new int[1];
-		boolean val = Widget.gtk_widget_intersect(handle, area.getHandle(), rect);
+		boolean val = Widget.gtk_widget_intersect(getHandle(), area.getHandle(), rect);
 		if (false == val)
 			return null;
 		return new Rectangle(rect[0]);
@@ -230,14 +235,14 @@ public class Widget extends GtkObject {
 	 * @return true if the widget has focus.
 	 */
 	public boolean hasFocus() {
-		return Widget.gtk_widget_is_focus(handle);
+		return Widget.gtk_widget_is_focus(getHandle());
 	}
 
 	/**
 	 * Causes the widget to gain focus within its' TOPLEVEL.
 	 */
 	public void grabFocus() {
-		Widget.gtk_widget_grab_focus(handle);
+		Widget.gtk_widget_grab_focus(getHandle());
 	}
 
 	/**
@@ -249,7 +254,7 @@ public class Widget extends GtkObject {
 		// make sure CAN_DEFAULT has been set.
 		//Moving flag setting to the c layer.
 		//GtkObject.setFlags(handle, WidgetFlags.CAN_DEFAULT.getValue());
-		Widget.gtk_widget_grab_default(handle);
+		Widget.gtk_widget_grab_default(getHandle());
 	}
 
 	/**
@@ -260,7 +265,7 @@ public class Widget extends GtkObject {
 	 * @param name The name for the widget.
 	 */
 	public void setName(String name) {
-		Widget.gtk_widget_set_name(handle, name);
+		Widget.gtk_widget_set_name(getHandle(), name);
 	}
 
 	/**
@@ -269,7 +274,7 @@ public class Widget extends GtkObject {
 	 * @return The name for the widget.
 	 */
 	public String getName() {
-		return Widget.gtk_widget_get_name(handle);
+		return Widget.gtk_widget_get_name(getHandle());
 	}
 
 	/**
@@ -278,7 +283,7 @@ public class Widget extends GtkObject {
 	 * @param sensitive Indicates if the widget should be sensitive.
 	 */
 	public void setSensitive(boolean sensitive) {
-		Widget.gtk_widget_set_sensitive(handle, sensitive);
+		Widget.gtk_widget_set_sensitive(getHandle(), sensitive);
 	}
 
 	/**
@@ -287,7 +292,7 @@ public class Widget extends GtkObject {
 	 * @return true if the widget is sensitive, false otherwise.
 	 */
     public boolean getSensitive() {
-		return gtk_widget_get_sensitive(handle);
+		return gtk_widget_get_sensitive(getHandle());
 	}
 
 	/**
@@ -296,8 +301,13 @@ public class Widget extends GtkObject {
 	 * @return The topmost widget.
 	 */
 	public Widget getToplevel() {
-		int newHandle = Widget.gtk_widget_get_toplevel(handle);
-		return new Widget(newHandle);
+		int hndl = gtk_widget_get_toplevel(getHandle());
+		if (0 == hndl)
+			return null;
+		GObject obj = retrieveGObject(hndl);
+		if (null != obj)
+			return (Widget)obj;
+		return new Widget(hndl);
 	}
 
 	/**
@@ -306,7 +316,10 @@ public class Widget extends GtkObject {
 	 * @return The colormap used for this widget.
 	 */
 	public Colormap getColormap() {
-		int colorHandle = Widget.gtk_widget_get_colormap(handle);
+		int colorHandle = Widget.gtk_widget_get_colormap(getHandle());
+		GObject obj = retrieveGObject(colorHandle);
+		if (null != obj)
+			return (Colormap)obj;
 		return new Colormap(colorHandle);
 	}
 
@@ -316,7 +329,7 @@ public class Widget extends GtkObject {
 	 * @param colormap The colormap to be used.
 	 */
 	public void setColormap(Colormap colormap) {
-		Widget.gtk_widget_set_colormap(handle, colormap.getHandle());
+		Widget.gtk_widget_set_colormap(getHandle(), colormap.getHandle());
 	}
 
 	/**
@@ -328,7 +341,7 @@ public class Widget extends GtkObject {
 	public Point getPointer() {
 		int[] x = new int[1];
 		int[] y = new int[1];
-		Widget.gtk_widget_get_pointer(handle, x, y);
+		Widget.gtk_widget_get_pointer(getHandle(), x, y);
 		if (null == x)
 			return null;
 		if (null == y)
@@ -342,7 +355,7 @@ public class Widget extends GtkObject {
 	 * @param ancestor The possible parent widget
 	 */
 	public boolean isAncestor(Widget ancestor) {
-		return Widget.gtk_widget_is_ancestor(handle, ancestor.getHandle());
+		return Widget.gtk_widget_is_ancestor(getHandle(), ancestor.getHandle());
 	}
 
 	/**
@@ -371,7 +384,7 @@ public class Widget extends GtkObject {
 	 * @param y Y Position of the shape mask with respect to the window.
 	 */
 	public void shapeCombineMask(Bitmap shapeMask, int x, int y) {
-		Widget.gtk_widget_shape_combine_mask(handle, shapeMask.getHandle(), x, y);
+		Widget.gtk_widget_shape_combine_mask(getHandle(), shapeMask.getHandle(), x, y);
 	}
 
 	/**
@@ -382,7 +395,7 @@ public class Widget extends GtkObject {
 	 * @param color The color to set the foreground.
 	 */
 	public void setForegroundColor(StateType state, Color color) {
-		Widget.gtk_widget_modify_fg(handle, state.getValue(), color.getHandle());
+		Widget.gtk_widget_modify_fg(getHandle(), state.getValue(), color.getHandle());
 	}
 
 	/**
@@ -393,7 +406,7 @@ public class Widget extends GtkObject {
 	 * @param color The color to set the background.
 	 */
 	public void setBackgroundColor(StateType state, Color color) {
-		Widget.gtk_widget_modify_bg(handle, state.getValue(), color.getHandle());
+		Widget.gtk_widget_modify_bg(getHandle(), state.getValue(), color.getHandle());
 	}
 
 	/**
@@ -405,7 +418,7 @@ public class Widget extends GtkObject {
 	 * @param color The color to set the text.
 	 */
 	public void setTextColor(StateType state, Color color) {
-		Widget.gtk_widget_modify_text(handle, state.getValue(), color.getHandle());
+		Widget.gtk_widget_modify_text(getHandle(), state.getValue(), color.getHandle());
 	}
 
 	/**
@@ -417,7 +430,7 @@ public class Widget extends GtkObject {
 	 * @param color The color to set.
 	 */
 	public void setBaseColor(StateType state, Color color) {
-		Widget.gtk_widget_modify_base(handle, state.getValue(), color.getHandle());
+		Widget.gtk_widget_modify_base(getHandle(), state.getValue(), color.getHandle());
 	}
 
 	/**
@@ -426,7 +439,7 @@ public class Widget extends GtkObject {
 	 * @param fontDesc The font description to use.
 	 */
 	public void setFont(FontDescription fontDesc) {
-		Widget.gtk_widget_modify_font(handle, fontDesc.getHandle());
+		Widget.gtk_widget_modify_font(getHandle(), fontDesc.getHandle());
 	}
 
 	/**
@@ -436,7 +449,7 @@ public class Widget extends GtkObject {
 	 * @param style The style to apply to the Widget
 	 */
 	public void modifyStyle(RcStyle style) {
-		Widget.gtk_widget_modify_style(handle, style.getHandle());
+		Widget.gtk_widget_modify_style(getHandle(), style.getHandle());
 	}
 
 	/**
@@ -445,7 +458,7 @@ public class Widget extends GtkObject {
 	 * @return The current RcStyle for the Widget.
 	 */
 	public RcStyle getModifierStyle() {
-		int hndl = Widget.gtk_widget_get_modifier_style(handle);
+		int hndl = Widget.gtk_widget_get_modifier_style(getHandle());
 		return new RcStyle(hndl);
 	}
 
@@ -455,7 +468,10 @@ public class Widget extends GtkObject {
 	 * @return The current Style for the Widget.
 	 */
 	public Style getStyle() {
-		int hndl = Widget.gtk_widget_get_style(handle);
+		int hndl = Widget.gtk_widget_get_style(getHandle());
+		GObject obj = retrieveGObject(hndl);
+		if (null != obj)
+			return (Style)obj;
 		return new Style(hndl);
 	}
 
@@ -466,7 +482,7 @@ public class Widget extends GtkObject {
 	 * @return A context that can be used for drawig text.
 	 */
 	public Context createContext() {
-		int pc = Widget.gtk_widget_create_pango_context(handle);
+		int pc = Widget.gtk_widget_create_pango_context(getHandle());
 		return new Context(pc);
 	}
 
@@ -479,7 +495,10 @@ public class Widget extends GtkObject {
 	 * @return A context that can be used for drawing text.
 	 */
 	public Context getContext() {
-		int pc = Widget.gtk_widget_get_pango_context(handle);
+		int pc = Widget.gtk_widget_get_pango_context(getHandle());
+		GObject obj = retrieveGObject(pc);
+		if (null != obj)
+			return (Context)obj;
 		return new Context(pc);
 	}
 
@@ -491,7 +510,7 @@ public class Widget extends GtkObject {
 	 * @return The layout that can be used for drawing text.
 	 */
 	public org.gnu.pango.Layout createLayout(String text) {
-		int layout = Widget.gtk_widget_create_pango_layout(handle, text);
+		int layout = Widget.gtk_widget_create_pango_layout(getHandle(), text);
 		return new org.gnu.pango.Layout(layout);
 	}
 
@@ -502,7 +521,7 @@ public class Widget extends GtkObject {
 	 * @param doubleBuffered Set if the widget should be double buffered.
 	 */
 	public void setDoubleBuffered(boolean doubleBuffered) {
-		Widget.gtk_widget_set_double_buffered(handle, doubleBuffered);
+		Widget.gtk_widget_set_double_buffered(getHandle(), doubleBuffered);
 	}
 
 	/**
@@ -511,28 +530,43 @@ public class Widget extends GtkObject {
 	 * @returns The AtkObject.
 	 */
 	public AtkObject getAccessible() {
-		int ao = Widget.gtk_widget_get_accessible(handle);
+		int ao = Widget.gtk_widget_get_accessible(getHandle());
+		GObject obj = retrieveGObject(ao);
+		if (null != obj)
+			return (AtkObject)obj;
 		return new AtkObject(ao);
 	}
 
 	public boolean canActivateAccel(int signalId) {
-		return gtk_widget_can_activate_accel(handle, signalId);
+		return gtk_widget_can_activate_accel(getHandle(), signalId);
 	}
 
 	public Screen getScreen() {
-		return new Screen(gtk_widget_get_screen(handle));
+		int hndl = gtk_widget_get_screen(getHandle());
+		GObject obj = retrieveGObject(hndl);
+		if (null != obj)
+			return (Screen)obj;
+		return new Screen(hndl);
 	}
 
 	public boolean hasScreen() {
-		return gtk_widget_has_screen(handle);
+		return gtk_widget_has_screen(getHandle());
 	}
 
 	public Display getDisplay() {
-		return new Display(gtk_widget_get_display(handle));
+		int hndl = gtk_widget_get_display(getHandle());
+		GObject obj = retrieveGObject(hndl);
+		if (null != obj)
+			return (Display)obj;
+		return new Display(hndl);
 	}
 
 	public Window getRootWindow() {
-		return new Window(gtk_widget_get_root_window(handle));
+		int hndl = gtk_widget_get_root_window(getHandle());
+		GObject obj = retrieveGObject(hndl);
+		if (null != obj)
+			return (Window)obj;
+		return new Window(hndl);
 	}
 
 	/**
@@ -556,7 +590,7 @@ public class Widget extends GtkObject {
 	 * @param height The minimum height size to use.
 	 */
 	public void setMinimumSize(int width, int height) {
-		Widget.gtk_widget_set_size_request(handle, width, height);
+		Widget.gtk_widget_set_size_request(getHandle(), width, height);
 	}
 
 	/**
@@ -564,15 +598,23 @@ public class Widget extends GtkObject {
 	 * propogate them to the listeners.
 	 */
 	protected void initializeEventHandlers() {
-		evtMap.initialize(this);
+		if (!eventsInitialized) {
+			evtMap.initialize(this);
+			eventsInitialized = true;
+		}
 	}
 
 	public Class getEventListenerClass(String signal) {
-		return evtMap.getListenerClass(signal);
+		Class rv = evtMap.getEventListenerClass(signal);
+		if (rv == null)
+			rv = super.getEventListenerClass( signal );
+		return rv;
 	}
 
 	public GtkEventType getEventType(String signal) {
-		return evtMap.getEventType(signal);
+		GtkEventType et = evtMap.getEventType(signal);
+		if (et == null) et = super.getEventType( signal );
+		return et;
 	}
 
 	private static EventMap evtMap = new EventMap();
@@ -904,7 +946,8 @@ public class Widget extends GtkObject {
 		try {
 			Class cls = Class.forName(widgetClass);
 			java.lang.reflect.Constructor constr = cls.getConstructor(new Class[] { Integer.TYPE });
-			return (Widget)constr.newInstance(new Object[] { new Integer(handle)});
+			Widget cons =  (Widget)constr.newInstance(new Object[] { new Integer(handle)});
+			return cons;
 		} catch (ClassNotFoundException e) {
 			throw e;
 		} catch (NoSuchMethodException e) {
@@ -944,14 +987,14 @@ public class Widget extends GtkObject {
 		int[] targets_g = new int[targets.length];
 		for (int i = 0; i < targets.length; i++)
 			targets_g[i] = targets[i].getHandle();
-		gtk_drag_source_set(handle, buttons.getValue(), targets_g, actions.getValue());
+		gtk_drag_source_set(getHandle(), buttons.getValue(), targets_g, actions.getValue());
 	}
 
 	/**
 	 * Removes support for this widget being a drag and drop source.
 	 */
 	public void setNoDragSource() {
-		gtk_drag_source_unset(handle);
+		gtk_drag_source_unset(getHandle());
 	}
 
 	public void setDragIcon(Widget widget, Colormap cmap, Pixmap pixmap, Bitmap mask) {
@@ -975,14 +1018,14 @@ public class Widget extends GtkObject {
 		int[] targets_g = new int[targets.length];
 		for (int i = 0; i < targets.length; i++)
 			targets_g[i] = targets[i].getHandle();
-		gtk_drag_dest_set(handle, flags.getValue(), targets_g, actions.getValue());
+		gtk_drag_dest_set(getHandle(), flags.getValue(), targets_g, actions.getValue());
 	}
 
 	/**
 	 * Removes support for this widget being a drag and drop target
 	 */
 	public void setNoDragDestination() {
-		gtk_drag_dest_unset(handle);
+		gtk_drag_dest_unset(getHandle());
 	}
 	public static void getData(Widget widget, DragContext context, int target) {
 		gtk_drag_get_data(widget.getHandle(), context.getHandle(), target);
@@ -1051,7 +1094,19 @@ public class Widget extends GtkObject {
 	}
 
 	protected void handleDragDestination(int context, int x, int y, int data, int id, int time) {
-		fireDragDestinationEvent(this, new org.gnu.gdk.DragContext(context), x, y, new SelectionData(data), id, time);
+		SelectionData sd;
+		GObject obj = retrieveGObject(data);
+		if (null != obj)
+			sd = (SelectionData)obj;
+		else
+			sd = new SelectionData(data);
+		DragContext dc;
+		GObject obj2 = retrieveGObject(context);
+		if (null != obj2)
+			dc = (DragContext)obj2;
+		else
+			dc = new DragContext(context);
+		fireDragDestinationEvent(this, dc, x, y, sd, id, time);
 	}
 
 	/**
@@ -1102,7 +1157,19 @@ public class Widget extends GtkObject {
 	}
 
 	protected void handleDragDataGet(int context, int data, int id, int time) {
-		fireDragSourceEvent(this, new org.gnu.gdk.DragContext(context), new SelectionData(data), id, time);
+		SelectionData sd;
+		GObject obj = retrieveGObject(data);
+		if (null != obj)
+			sd = (SelectionData)obj;
+		else
+			sd = new SelectionData(data);
+		DragContext dc;
+		GObject obj2 = retrieveGObject(context);
+		if (null != obj2)
+			dc = (DragContext)obj2;
+		else
+			dc = new DragContext(context);
+		fireDragSourceEvent(this, dc, sd, id, time);
 	}
 
 	/****************************************
